package se.sjv.integration.test.trace;

import java.util.LinkedList;
import java.util.List;

import javax.inject.Inject;
import javax.inject.Named;

import org.slf4j.Logger;
import org.switchyard.Exchange;
import org.switchyard.ExchangeInterceptor;
import org.switchyard.HandlerException;

/**
 * Används för att tillhandahålla spårning av meddelande skickade i Switchyard. Loggar meddelanden (till info), utan
 * body, före och efter provider och consumer som default. För att ändra vad som loggas kan klassen
 * se.sjv.integration.switchyard.common.tracer.MessageTracerParametersImpl skapas, den behöver då implementera
 * interfacet MessageTracerParameters för att tillhandahålla avvikande loggningsbetende avseende loggning av provider,
 * consumer och body.
 * 
 * <pre>
 * <code>
 * package se.sjv.integration.switchyard.common.tracer;
 * 
 * public class MessageTracerParametersImpl implements MessageTracerParameters {
 * 
 *     @Override
 *     public boolean getPrintBody() {
 *         return true;
 *     }
 * 
 *     @Override
 *     public boolean getPrintProvider() {
 *         return true;
 *     }
 * 
 *     @Override
 *     public boolean getPrintConsumer() {
 *         return true;
 *     }
 * }
 * </code>
 * </pre>
 */
@Named("TraceMessage")
public class MessageTracer implements ExchangeInterceptor {

    private final List<String> providers = new LinkedList<String>();
    private final boolean printBody;

    @Inject
    private Logger logger;

    @Inject
    private ExchangeFormatter exchangeFormatter;

    /**
     * Kontrollerar om det finns någon se.sjv.integration.switchyard.common.tracer.MessageTracerParametersImpl för att
     * sätta vad som ska loggas, i annat fall sätts defaultvärden.
     */
    public MessageTracer() {
        this("se.sjv.integration.switchyard.common.tracer.MessageTracerParametersImpl");
    }

    MessageTracer(final String parameterClassName) {
        printBody = false;
        providers.add(PROVIDER);
        providers.add(CONSUMER);
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public void before(final String target, final Exchange exchange) throws HandlerException {
        if (logger.isDebugEnabled()) {
            logger.info(getSwitchyardFormatterOutput(exchange));
        } else {
            logger.info(exchangeFormatter.format(exchange, shouldPrintBody()));
        }
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public void after(final String target, final Exchange exchange) throws HandlerException {
        if (logger.isDebugEnabled()) {
            logger.info(getSwitchyardFormatterOutput(exchange));
        } else {
            logger.info(exchangeFormatter.format(exchange, shouldPrintBody()));
        }
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public List<String> getTargets() {
        return providers;
    }

    boolean shouldPrintBody() {
        return printBody;
    }

    String getSwitchyardFormatterOutput(final Exchange exchange) {
        return org.switchyard.runtime.util.ExchangeFormatter.format(exchange, true);
    }
}
